@@ -64,6 +64,14 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
+/*
+/*	Wietse Venema
+/*	porcupine.org
 /*--*/
 
 /* System library. */
@@ -181,10 +189,10 @@ void    cleanup_out_header(CLEANUP_STATE *state, VSTRING *header_buf)
      * Fix 20140711: Auto-detect the presence of a non-ASCII header.
      */
     if (var_smtputf8_enable && *STR(header_buf) && !allascii(STR(header_buf))) {
-	state->smtputf8 |= SMTPUTF8_FLAG_HEADER;
+	state->sendopts |= SMTPUTF8_FLAG_HEADER;
 	/* Fix 20140713: request SMTPUTF8 support selectively. */
 	if (state->flags & CLEANUP_FLAG_AUTOUTF8)
-	    state->smtputf8 |= SMTPUTF8_FLAG_REQUESTED;
+	    state->sendopts |= SMTPUTF8_FLAG_REQUESTED;
     }
 
     /*
