@@ -7,31 +7,31 @@
 /*	#include <post_mail.h>
 /*
 /*	VSTREAM	*post_mail_fopen(sender, recipient, source_class, trace_flags,
-/*		utf8_flags, queue_id)
+/*		sendopts, queue_id)
 /*	const char *sender;
 /*	const char *recipient;
 /*	int	source_class;
 /*	int	trace_flags;
-/*	int	utf8_flags;
+/*	int	sendopts;
 /*	VSTRING *queue_id;
 /*
 /*	VSTREAM	*post_mail_fopen_nowait(sender, recipient, source_class,
-/*					trace_flags, utf8_flags, queue_id)
+/*					trace_flags, sendopts, queue_id)
 /*	const char *sender;
 /*	const char *recipient;
 /*	int	source_class;
 /*	int	trace_flags;
-/*	int	utf8_flags;
+/*	int	sendopts;
 /*	VSTRING *queue_id;
 /*
 /*	void	post_mail_fopen_async(sender, recipient, source_class,
-/*					trace_flags, utf8_flags,
+/*					trace_flags, sendopts,
 /*					queue_id, notify, context)
 /*	const char *sender;
 /*	const char *recipient;
 /*	int	source_class;
 /*	int	trace_flags;
-/*	int	utf8_flags;
+/*	int	sendopts;
 /*	VSTRING *queue_id;
 /*	void	(*notify)(VSTREAM *stream, void *context);
 /*	void	*context;
@@ -116,9 +116,9 @@
 /*	autodetection.
 /* .IP trace_flags
 /*	Message tracing flags as specified in \fB<deliver_request.h>\fR.
-/* .IP utf8_flags
-/*	Flags defined in <smtputf8.h>. Flags other than
-/*	SMTPUTF8_FLAG_REQUESTED are ignored.
+/* .IP sendopts
+/*	Flags defined in <sendopts.h>. This ignores flags based on
+/*	message header content, or envelope email addresses.
 /* .IP queue_id
 /*	Null pointer, or pointer to buffer that receives the queue
 /*	ID of the new message.
@@ -195,7 +195,7 @@ typedef struct {
     char   *recipient;
     int     source_class;
     int     trace_flags;
-    int     utf8_flags;
+    int     sendopts;
     POST_MAIL_NOTIFY notify;
     void   *context;
     VSTREAM *stream;
@@ -217,7 +217,7 @@ typedef struct {
 static void post_mail_init(VSTREAM *stream, const char *sender,
 			           const char *recipient,
 			           int source_class, int trace_flags,
-			           int utf8_flags, VSTRING *queue_id)
+			           int sendopts, VSTRING *queue_id)
 {
     VSTRING *id = queue_id ? queue_id : vstring_alloc(100);
     struct timeval now;
@@ -225,14 +225,25 @@ static void post_mail_init(VSTREAM *stream, const char *sender,
     int     cleanup_flags =
     int_filt_flags(source_class) | CLEANUP_FLAG_MASK_INTERNAL
     | smtputf8_autodetect(source_class)
-    | ((utf8_flags & SMTPUTF8_FLAG_REQUESTED) ? CLEANUP_FLAG_SMTPUTF8 : 0);
+    | ((sendopts & SMTPUTF8_FLAG_REQUESTED) ? CLEANUP_FLAG_SMTPUTF8 : 0);
+    /* TODO(wietse) REQUIRETLS. */
 
     GETTIMEOFDAY(&now);
     date = mail_date(now.tv_sec);
 
     /*
-     * XXX Don't flush buffers while sending the initial message records.
-     * That would cause deadlock between verify(8) and cleanup(8) servers.
+     * The comment in the next paragraph is likely obsolete. Fix 20030610
+     * changed the verify server to use asynchronous submission of mail
+     * probes, to avoid blocking the post_mail client for in_flow_delay
+     * seconds when the cleanup service receives email messages faster than
+     * they are delivered. Instead, the post_mail client waits until the
+     * cleanup server announces its availability to receive input. A similar
+     * change was made at the end of submission, to avoid blocking the
+     * post_mail client for up to trigger_timeout seconds when the cleanup
+     * server attempts to notify a queue manager that is overwhelmed.
+     * 
+     * XXX Don't flush buffers while sending the initial message records. That
+     * would cause deadlock between verify(8) and cleanup(8) servers.
      */
     vstream_control(stream, VSTREAM_CTL_BUFSIZE, 2 * VSTREAM_BUFSIZE,
 		    VSTREAM_CTL_END);
@@ -241,6 +252,7 @@ static void post_mail_init(VSTREAM *stream, const char *sender,
      * Negotiate with the cleanup service. Give up if we can't agree.
      */
     if (attr_scan(stream, ATTR_FLAG_STRICT,
+		  RECV_ATTR_STREQ(MAIL_ATTR_PROTO, MAIL_ATTR_PROTO_CLEANUP),
 		  RECV_ATTR_STR(MAIL_ATTR_QUEUEID, id),
 		  ATTR_TYPE_END) != 1
 	|| attr_print(stream, ATTR_FLAG_NONE,
@@ -278,13 +290,13 @@ static void post_mail_init(VSTREAM *stream, const char *sender,
 
 VSTREAM *post_mail_fopen(const char *sender, const char *recipient,
 			         int source_class, int trace_flags,
-			         int utf8_flags, VSTRING *queue_id)
+			         int sendopts, VSTRING *queue_id)
 {
     VSTREAM *stream;
 
     stream = mail_connect_wait(MAIL_CLASS_PUBLIC, var_cleanup_service);
     post_mail_init(stream, sender, recipient, source_class, trace_flags,
-		   utf8_flags, queue_id);
+		   sendopts, queue_id);
     return (stream);
 }
 
@@ -292,14 +304,14 @@ VSTREAM *post_mail_fopen(const char *sender, const char *recipient,
 
 VSTREAM *post_mail_fopen_nowait(const char *sender, const char *recipient,
 				        int source_class, int trace_flags,
-				        int utf8_flags, VSTRING *queue_id)
+				        int sendopts, VSTRING *queue_id)
 {
     VSTREAM *stream;
 
     if ((stream = mail_connect(MAIL_CLASS_PUBLIC, var_cleanup_service,
 			       BLOCKING)) != 0)
 	post_mail_init(stream, sender, recipient, source_class, trace_flags,
-		       utf8_flags, queue_id);
+		       sendopts, queue_id);
     else
 	msg_warn("connect to %s/%s: %m",
 		 MAIL_CLASS_PUBLIC, var_cleanup_service);
@@ -328,7 +340,7 @@ static void post_mail_open_event(int event, void *context)
 	non_blocking(vstream_fileno(state->stream), BLOCKING);
 	post_mail_init(state->stream, state->sender,
 		       state->recipient, state->source_class,
-		       state->trace_flags, state->utf8_flags,
+		       state->trace_flags, state->sendopts,
 		       state->queue_id);
 	myfree(state->sender);
 	myfree(state->recipient);
@@ -380,7 +392,7 @@ static void post_mail_open_event(int event, void *context)
 
 void    post_mail_fopen_async(const char *sender, const char *recipient,
 			              int source_class, int trace_flags,
-			              int utf8_flags, VSTRING *queue_id,
+			              int sendopts, VSTRING *queue_id,
 			              void (*notify) (VSTREAM *, void *),
 			              void *context)
 {
@@ -393,7 +405,7 @@ void    post_mail_fopen_async(const char *sender, const char *recipient,
     state->recipient = mystrdup(recipient);
     state->source_class = source_class;
     state->trace_flags = trace_flags;
-    state->utf8_flags = utf8_flags;
+    state->sendopts = sendopts;
     state->notify = notify;
     state->context = context;
     state->stream = stream;
