@@ -50,6 +50,9 @@
 /*	Google, Inc.
 /*	111 8th Avenue
 /*	New York, NY 10011, USA
+/*
+/*	Wietse Venema
+/*	porcupine.org
 /*--*/
 
 /* System library. */
@@ -152,7 +155,7 @@ static int qmgr_deliver_send_request(QMGR_ENTRY *entry, VSTREAM *stream)
     MSG_STATS stats;
     char   *sender;
     int     flags;
-    int     smtputf8 = message->smtputf8;
+    int     sendopts = message->sendopts;
     const char *addr;
 
     /*
@@ -160,10 +163,10 @@ static int qmgr_deliver_send_request(QMGR_ENTRY *entry, VSTREAM *stream)
      */
     for (recipient = list.info; recipient < list.info + list.len; recipient++)
 	if (var_smtputf8_enable && (addr = recipient->address)[0]
-	    && !allascii(addr) && valid_utf8_string(addr, strlen(addr))) {
-	    smtputf8 |= SMTPUTF8_FLAG_RECIPIENT;
+	    && !allascii(addr) && valid_utf8_stringz(addr)) {
+	    sendopts |= SMTPUTF8_FLAG_RECIPIENT;
 	    if (message->verp_delims)
-		smtputf8 |= SMTPUTF8_FLAG_SENDER;
+		sendopts |= SMTPUTF8_FLAG_SENDER;
 	}
 
     /*
@@ -192,7 +195,7 @@ static int qmgr_deliver_send_request(QMGR_ENTRY *entry, VSTREAM *stream)
 	       SEND_ATTR_LONG(MAIL_ATTR_SIZE, message->cont_length),
 	       SEND_ATTR_STR(MAIL_ATTR_NEXTHOP, entry->queue->nexthop),
 	       SEND_ATTR_STR(MAIL_ATTR_ENCODING, message->encoding),
-	       SEND_ATTR_INT(MAIL_ATTR_SMTPUTF8, smtputf8),
+	       SEND_ATTR_INT(MAIL_ATTR_SENDOPTS, sendopts),
 	       SEND_ATTR_STR(MAIL_ATTR_SENDER, sender),
 	       SEND_ATTR_STR(MAIL_ATTR_DSN_ENVID, message->dsn_envid),
 	       SEND_ATTR_INT(MAIL_ATTR_DSN_RET, message->dsn_ret),
@@ -339,7 +342,7 @@ static void qmgr_deliver_update(int unused_event, void *context)
 #define SUSPENDED	"delivery temporarily suspended: "
 
     if (status == DELIVER_STAT_CRASH)
-	DSN_SIMPLE(&dsb->dsn, "4.3.0", "unknown mail transport error");
+	(void) DSN_SIMPLE(&dsb->dsn, "4.3.0", "unknown mail transport error");
     if (status == DELIVER_STAT_CRASH || status == DELIVER_STAT_DEFER) {
 	message->flags |= DELIVER_STAT_DEFER;
 	if (VSTRING_LEN(dsb->status)) {
