@@ -98,6 +98,9 @@
 /*	111 8th Avenue
 /*	New York, NY 10011, USA
 /*
+/*	Wietse Venema
+/*	porcupine.org
+/*
 /*	Preemptive scheduler enhancements:
 /*	Patrik Rak
 /*	Modra 6
@@ -146,7 +149,7 @@
 #include <split_addr.h>
 #include <dsn_mask.h>
 #include <rec_attr_map.h>
-#include <smtputf8.h>
+#include <sendopts.h>
 
 /* Client stubs. */
 
@@ -190,7 +193,7 @@ static QMGR_MESSAGE *qmgr_message_create(const char *queue_name,
     message->sender = 0;
     message->dsn_envid = 0;
     message->dsn_ret = 0;
-    message->smtputf8 = 0;
+    message->sendopts = 0;
     message->filter_xport = 0;
     message->inspect_xport = 0;
     message->redirect_addr = 0;
@@ -590,7 +593,7 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 				 &message->data_size, &message->data_offset,
 				    &message->rcpt_unread, &message->rflags,
 				    &message->cont_length,
-				    &message->smtputf8)) >= 3) {
+				    &message->sendopts)) >= 3) {
 		    /* Postfix >= 1.0 (a.k.a. 20010228). */
 		    if (message->data_offset <= 0 || message->data_size <= 0) {
 			msg_warn("%s: invalid size record: %.100s",
@@ -605,7 +608,7 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 			break;
 		    }
 		    /* Forward compatibility. */
-		    message->smtputf8 &= SMTPUTF8_FLAG_ALL;
+		    message->sendopts &= SOPT_FLAG_ALL;
 		} else if (count == 1) {
 		    /* Postfix < 1.0 (a.k.a. 20010228). */
 		    qmgr_message_oldstyle_scan(message);
