@@ -102,6 +102,7 @@
 #include <msg.h>
 #include <mymalloc.h>
 #include <vstring.h>
+#include <sane_strtol.h>
 #include <stringops.h>
 #include <valid_hostname.h>
 #include <valid_utf8_hostname.h>
@@ -113,6 +114,10 @@
 #include <maps.h>
 #include <dsn_buf.h>
 
+/* TLS library. */
+
+#include <tlsrpt_wrapper.h>
+
 /* DNS library. */
 
 #include <dns.h>
@@ -221,15 +226,21 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
 {
     const char *lookup;
     char   *policy;
-    char   *saved_policy;
+    char   *saved_policy = 0;
     char   *tok;
-    const char *err;
     char   *name;
     char   *val;
     static VSTRING *cbuf;
+    char   *free_me = 0;
 
 #undef FREE_RETURN
-#define FREE_RETURN do { myfree(saved_policy); return; } while (0)
+#define FREE_RETURN do { \
+	if (saved_policy) \
+	    myfree(saved_policy); \
+	if (free_me) \
+	    myfree(free_me); \
+	return; \
+    } while (0)
 
 #define INVALID_RETURN(why, levelp) do { \
 	    MARK_INVALID((why), (levelp)); FREE_RETURN; } while (0)
@@ -250,7 +261,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
     }
     saved_policy = policy = mystrdup(lookup);
 
-    if ((tok = mystrtok(&policy, CHARS_COMMA_SP)) == 0) {
+    if ((tok = mystrtokq(&policy, CHARS_COMMA_SP, CHARS_BRACE)) == 0) {
 	msg_warn("%s: invalid empty policy", WHERE);
 	INVALID_RETURN(tls->why, site_level);
     }
@@ -265,7 +276,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
      * Warn about ignored attributes when TLS is disabled.
      */
     if (*site_level < TLS_LEV_MAY) {
-	while ((tok = mystrtok(&policy, CHARS_COMMA_SP)) != 0)
+	while ((tok = mystrtokq(&policy, CHARS_COMMA_SP, CHARS_BRACE)) != 0)
 	    msg_warn("%s: ignoring attribute \"%s\" with TLS disabled",
 		     WHERE, tok);
 	FREE_RETURN;
@@ -274,9 +285,18 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
     /*
      * Errors in attributes may have security consequences, don't ignore
      * errors that can degrade security.
+     * 
+     * Caution: normalize whitespace, to neutralize line break etc. characters
+     * inside the value portion of { name = value }.
      */
-    while ((tok = mystrtok(&policy, CHARS_COMMA_SP)) != 0) {
-	if ((err = split_nameval(tok, &name, &val)) != 0) {
+    while ((tok = mystrtokq(&policy, CHARS_COMMA_SP, CHARS_BRACE)) != 0) {
+	const char *err;
+
+#define EXTPAR_OPT	(EXTPAR_FLAG_STRIP | EXTPAR_FLAG_NORMAL_WS)
+
+	if ((tok[0] == CHARS_BRACE[0]
+	     && (err = free_me = extpar(&tok, CHARS_BRACE, EXTPAR_OPT)) != 0)
+	    || (err = split_nameval(tok, &name, &val)) != 0) {
 	    msg_warn("%s: malformed attribute/value pair \"%s\": %s",
 		     WHERE, tok, err);
 	    INVALID_RETURN(tls->why, site_level);
@@ -391,6 +411,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
 	    }
 	    continue;
 	}
+	/* Last one wins. */
 	if (!strcasecmp(name, "enable_rpk")) {
 	    /* Ultimately ignored at some security levels */
 	    if (strcasecmp(val, "yes") == 0) {
@@ -404,10 +425,98 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
 	    }
 	    continue;
 	}
+	/* Only one instance per policy. */
+	if (!strcasecmp(name, EXT_POLICY_TTL)) {
+	    char   *end;
+	    long    lval;
+
+	    if (tls->ext_policy_ttl != EXT_POLICY_TTL_UNSET) {
+		msg_warn("%s: attribute \"%s\" is specified multiple times",
+			 WHERE, name);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    if (!alldig(val) || ((lval = sane_strtol(val, &end, 10)),
+				 ((tls->ext_policy_ttl = lval) != lval))
+		|| *end != 0) {
+		msg_warn("%s: attribute \"%s\" has a malformed value: \"%s\"",
+			 WHERE, name, val);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    continue;
+	}
+	/* Only one instance per policy. */
+	if (!strcasecmp(name, EXT_POLICY_TYPE)) {
+	    if (tls->ext_policy_type) {
+		msg_warn("%s: attribute \"%s\" is specified multiple times",
+			 WHERE, name);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    if (!valid_tlsrpt_policy_type(val)) {
+		msg_warn("%s: attribute \"%s\" has an unexpected value: \"%s\"",
+			 WHERE, name, val);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    tls->ext_policy_type = mystrdup(val);
+	    continue;
+	}
+	/* Only one instance per policy. */
+	if (!strcasecmp(name, EXT_POLICY_DOMAIN)) {
+	    if (tls->ext_policy_domain) {
+		msg_warn("%s: attribute \"%s\" is specified multiple times",
+			 WHERE, name);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    if (!valid_hostname(val, DO_GRIPE)) {
+		msg_warn("%s: attribute \"%s\" has a malformed value: \"%s\"",
+			 WHERE, name, val);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    tls->ext_policy_domain = mystrdup(val);
+	    continue;
+	}
+	/* Multiple instances per policy are allowed. */
+	if (!strcasecmp(name, EXT_POLICY_STRING)) {
+	    if (tls->ext_policy_strings == 0)
+		tls->ext_policy_strings = argv_alloc(1);
+	    argv_add(tls->ext_policy_strings, val, (char *) 0);
+	    continue;
+	}
+	/* Multiple instances per policy are allowed. */
+	if (!strcasecmp(name, EXT_MX_HOST_PATTERN)) {
+	    if (tls->ext_mx_host_patterns == 0)
+		tls->ext_mx_host_patterns = argv_alloc(1);
+	    argv_add(tls->ext_mx_host_patterns, val, (char *) 0);
+	    continue;
+	}
+	/* Only one instance per policy. */
+	if (!strcasecmp(name, EXT_POLICY_FAILURE)) {
+	    if (tls->ext_policy_failure != 0) {
+		msg_warn("%s: attribute \"%s\" is specified multiple times",
+			 WHERE, name);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    if (!valid_tlsrpt_policy_failure(val)) {
+		msg_warn("%s: attribute \"%s\" has an unexpected value: \"%s\"",
+			 WHERE, name, val);
+		INVALID_RETURN(tls->why, site_level);
+	    }
+	    tls->ext_policy_failure = mystrdup(val);
+	    continue;
+	}
 	msg_warn("%s: invalid attribute name: \"%s\"", WHERE, name);
 	INVALID_RETURN(tls->why, site_level);
     }
-
+    if (tls->ext_policy_type == 0) {
+	if (tls->ext_policy_ttl != EXT_POLICY_TTL_UNSET
+	    || tls->ext_policy_strings
+	    || tls->ext_policy_domain || tls->ext_mx_host_patterns
+	    || tls->ext_policy_failure) {
+	    msg_warn("%s: built-in policy has unexpected attribute "
+		     "policy_ttl, policy_domain, policy_string, "
+		     "mx_host_pattern or policy_failure", WHERE);
+	    INVALID_RETURN(tls->why, site_level);
+	}
+    }
     FREE_RETURN;
 }
 
@@ -707,6 +816,16 @@ static void policy_delete(void *item, void *unused_context)
     if (tls->dane)
 	tls_dane_free(tls->dane);
     dsb_free(tls->why);
+    if (tls->ext_policy_type)
+	myfree(tls->ext_policy_type);
+    if (tls->ext_policy_domain)
+	myfree(tls->ext_policy_domain);
+    if (tls->ext_policy_strings)
+	argv_free(tls->ext_policy_strings);
+    if (tls->ext_mx_host_patterns)
+	argv_free(tls->ext_mx_host_patterns);
+    if (tls->ext_policy_failure)
+	myfree(tls->ext_policy_failure);
 
     myfree((void *) tls);
 }
