@@ -11,8 +11,9 @@
 /*	int	ok;
 /*	X509_STORE_CTX *ctx;
 /*
-/*	int     tls_log_verify_error(TLScontext)
+/*	int     tls_log_verify_error(TLScontext, tlsrpt)
 /*	TLS_SESS_STATE *TLScontext;
+/*	struct TLSRPT_WRAPPER *tlsrpt;
 /*
 /*	char *tls_peer_CN(peercert, TLScontext)
 /*	X509   *peercert;
@@ -89,6 +90,9 @@
 /*
 /*	Victor Duchovni
 /*	Morgan Stanley
+/*
+/*	Wietse Venema
+/*	porcupine.org
 /*--*/
 
 /* System library. */
@@ -107,6 +111,10 @@
 
 /* TLS library. */
 
+#ifdef USE_TLSRPT
+#include <tlsrpt_wrapper.h>
+#endif
+
 #define TLS_INTERNAL
 #include <tls.h>
 
@@ -194,18 +202,50 @@ int     tls_verify_certificate_callback(int ok, X509_STORE_CTX *ctx)
 
 /* tls_log_verify_error - Report final verification error status */
 
-void    tls_log_verify_error(TLS_SESS_STATE *TLScontext)
+void    tls_log_verify_error(TLS_SESS_STATE *TLScontext,
+			             struct TLSRPT_WRAPPER *tlsrpt)
 {
     char    buf[CCERT_BUFSIZ];
     int     err = TLScontext->errorcode;
     X509   *cert = TLScontext->errorcert;
     int     depth = TLScontext->errordepth;
 
+#ifdef USE_TLSRPT
+    VSTRING *err_vstr = vstring_alloc(100);
+
+#define CERT_ERROR_TO_STRING(err) \
+    translit(vstring_str(vstring_strcpy(err_vstr, \
+					X509_verify_cert_error_string(err))), \
+	     " ", "_")
+#endif
+
 #define PURPOSE ((depth>0) ? "CA": TLScontext->am_server ? "client": "server")
 
     if (err == X509_V_OK)
 	return;
 
+    /*
+     * If an external policy flagged an error, report that instead.
+     */
+    if (TLScontext->ffail_type) {
+	msg_info("certificate verification failed for %s: "
+		 "external policy failure (%s)",
+		 TLScontext->namaddr, TLScontext->ffail_type);
+#ifdef USE_TLSRPT
+	if (tlsrpt) {
+	    tlsrpt_failure_t failure_type;
+
+	    if ((failure_type = convert_tlsrpt_policy_failure(TLScontext->ffail_type)) < 0)
+		msg_panic("tls_log_verify_error: unexpected failure_reason: %s",
+			  TLScontext->ffail_type);
+	    trw_report_failure(tlsrpt, failure_type,
+			        /* additional_info= */ (char *) 0,
+			        /* failure_reason= */ (char *) 0);
+	}
+#endif
+	return;
+    }
+
     /*
      * Specific causes for verification failure.
      */
@@ -218,10 +258,22 @@ void    tls_log_verify_error(TLS_SESS_STATE *TLScontext)
 	 */
 	msg_info("certificate verification failed for %s: "
 		 "not trusted by local or TLSA policy", TLScontext->namaddr);
+#ifdef USE_TLSRPT
+	if (tlsrpt)
+	    trw_report_failure(tlsrpt, TLSRPT_CERTIFICATE_NOT_TRUSTED,
+			        /* additional_info= */ (char *) 0,
+			        /* failure_code= */ (char *) 0);
+#endif
 	break;
     case X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT:
 	msg_info("certificate verification failed for %s: "
 		 "self-signed certificate", TLScontext->namaddr);
+#ifdef USE_TLSRPT
+	if (tlsrpt)
+	    trw_report_failure(tlsrpt, TLSRPT_VALIDATION_FAILURE,
+			        /* additional_info= */ (char *) 0,
+			       CERT_ERROR_TO_STRING(err));
+#endif
 	break;
     case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
     case X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN:
@@ -237,25 +289,55 @@ void    tls_log_verify_error(TLS_SESS_STATE *TLScontext)
 	    strcpy(buf, "<unknown>");
 	msg_info("certificate verification failed for %s: untrusted issuer %s",
 		 TLScontext->namaddr, printable(buf, '?'));
+#ifdef USE_TLSRPT
+	if (tlsrpt)
+	    trw_report_failure(tlsrpt, TLSRPT_VALIDATION_FAILURE,
+			        /* additional_info= */ (char *) 0,
+			       CERT_ERROR_TO_STRING(err));
+#endif
 	break;
     case X509_V_ERR_CERT_NOT_YET_VALID:
     case X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD:
 	msg_info("%s certificate verification failed for %s: certificate not"
 		 " yet valid", PURPOSE, TLScontext->namaddr);
+#ifdef USE_TLSRPT
+	if (tlsrpt)
+	    trw_report_failure(tlsrpt, TLSRPT_VALIDATION_FAILURE,
+			        /* additional_info= */ (char *) 0,
+			       CERT_ERROR_TO_STRING(err));
+#endif
 	break;
     case X509_V_ERR_CERT_HAS_EXPIRED:
     case X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD:
 	msg_info("%s certificate verification failed for %s: certificate has"
 		 " expired", PURPOSE, TLScontext->namaddr);
+#ifdef USE_TLSRPT
+	if (tlsrpt)
+	    trw_report_failure(tlsrpt, TLSRPT_CERTIFICATE_EXPIRED,
+			        /* additional_info= */ (char *) 0,
+			        /* failure_code= */ (char *) 0);
+#endif
 	break;
     case X509_V_ERR_INVALID_PURPOSE:
 	msg_info("certificate verification failed for %s: not designated for "
 		 "use as a %s certificate", TLScontext->namaddr, PURPOSE);
+#ifdef USE_TLSRPT
+	if (tlsrpt)
+	    trw_report_failure(tlsrpt, TLSRPT_VALIDATION_FAILURE,
+			        /* additional_info= */ (char *) 0,
+			       CERT_ERROR_TO_STRING(err));
+#endif
 	break;
     case X509_V_ERR_CERT_CHAIN_TOO_LONG:
 	msg_info("certificate verification failed for %s: "
 		 "certificate chain longer than limit(%d)",
 		 TLScontext->namaddr, depth - 1);
+#ifdef USE_TLSRPT
+	if (tlsrpt)
+	    trw_report_failure(tlsrpt, TLSRPT_VALIDATION_FAILURE,
+			        /* additional_info= */ (char *) 0,
+			       CERT_ERROR_TO_STRING(err));
+#endif
 	break;
     default:
 	msg_info("%s certificate verification failed for %s: num=%d:%s",
@@ -263,6 +345,9 @@ void    tls_log_verify_error(TLS_SESS_STATE *TLScontext)
 		 X509_verify_cert_error_string(err));
 	break;
     }
+#ifdef USE_TLSRPT
+    vstring_free(err_vstr);
+#endif
 }
 
 #ifndef DONT_GRIPE
