@@ -200,7 +200,7 @@ char   *casefoldx(int flags, VSTRING *dest, const char *src, ssize_t len)
 	space_needed = ucasemap_utf8FoldCase(csm, STR(dest) + old_len,
 				     vstring_avail(dest), src, len, &error);
 	if (U_SUCCESS(error)) {
-	    VSTRING_AT_OFFSET(dest, old_len + space_needed);
+	    vstring_set_payload_size(dest, old_len + space_needed);
 	    if (vstring_avail(dest) == 0)	/* exact fit, no terminator */
 		VSTRING_TERMINATE(dest);	/* add terminator */
 	    break;
@@ -300,7 +300,7 @@ int     main(int argc, char **argv)
 		    encode_utf8(buffer, codepoint);
 		    if (msg_verbose)
 			vstream_printf("U+%X -> %s\n", codepoint, STR(buffer));
-		    if (valid_utf8_string(STR(buffer), LEN(buffer)) == 0)
+		    if (valid_utf8_stringz(STR(buffer)) == 0)
 			msg_fatal("bad utf-8 encoding for U+%X\n", codepoint);
 		    casefold(dest, STR(buffer));
 		}
