@@ -21,10 +21,6 @@
   */
 #define DICT_TYPE_PIPE	"pipemap"
 
-#ifdef SNAPSHOT
-#define DICT_TYPE_PIPE_LEGACY	"pipeline"
-#endif
-
 extern DICT *dict_pipe_open(const char *, int, int);
 
 /* LICENSE
