@@ -11,6 +11,12 @@
 /* DESCRIPTION
 /*	.nf
 
+/*
+ * System library.
+ */
+#include <stdarg.h>
+#include <time.h>
+
 /*
  * External interface.
  */
@@ -25,19 +31,30 @@ extern NORETURN PRINTFLIKE(1, 2) msg_fatal(const char *,...);
 extern NORETURN PRINTFLIKE(2, 3) msg_fatal_status(int, const char *,...);
 extern NORETURN PRINTFLIKE(1, 2) msg_panic(const char *,...);
 
+extern void vmsg_info(const char *, va_list);
+extern void vmsg_warn(const char *, va_list);
+extern void vmsg_error(const char *, va_list);
+extern NORETURN vmsg_fatal(const char *, va_list);
+extern NORETURN vmsg_fatal_status(int, const char *, va_list);
+extern NORETURN vmsg_panic(const char *, va_list);
+
 extern int msg_error_limit(int);
 extern void msg_error_clear(void);
 extern MSG_CLEANUP_FN msg_cleanup(MSG_CLEANUP_FN);
 
+extern void PRINTFLIKE(4, 5) msg_rate_delay(time_t *, int,
+	              void PRINTFPTRLIKE(1, 2) (*log_fn) (const char *,...),
+					            const char *,...);
+
 /* LICENSE
 /* .ad
 /* .fi
 /*	The Secure Mailer license must be distributed with this software.
 /* AUTHOR(S)
-/*      Wietse Venema
-/*      IBM T.J. Watson Research
-/*      P.O. Box 704
-/*      Yorktown Heights, NY 10598, USA
+/*	Wietse Venema
+/*	IBM T.J. Watson Research
+/*	P.O. Box 704
+/*	Yorktown Heights, NY 10598, USA
 /*--*/
 
 #endif
